/*      mod_graph.c      - 
**
**
** Copyright (c) 1997  Hughes Technologies Pty Ltd.
**
** Permission to use, copy, and distribute for non-commercial purposes,
** is hereby granted without fee, providing that the above copyright
** notice appear in all copies and that both the copyright notice and this
** permission notice appear in supporting documentation.
**
** The software may be modified for your own purposes, but modified versions
** may not be distributed.
**
** This software is provided "as is" without any expressed or implied warranty.
**
*/


#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <sys/types.h>
#include "lite.h"

#include "gd1.2/gd.h"
#include "gd1.2/gdfontt.h"
#include "gd1.2/gdfonts.h"
#include "gd1.2/gdfontmb.h"
#include "gd1.2/gdfontl.h"
#include "gd1.2/gdfontg.h"

#define MOD_GRAPH_SOURCE
#include "mod_graph.h"

extern  sym_t   *externReturn;



/*************************************************************************
**									**
**			Graph Library Setup				**
**									**
*************************************************************************/


int init_mod(modPtr)
	void	*modPtr;
{
	symCreateIntMacro("HTG_BORDER_NONE",HTG_BORDER_NONE);
	symCreateIntMacro("HTG_BORDER_BEVEL_IN",HTG_BORDER_BEVEL_IN);
	symCreateIntMacro("HTG_BORDER_BEVEL_OUT",HTG_BORDER_BEVEL_OUT);
	symCreateIntMacro("HTG_BORDER_FRAME_IN",HTG_BORDER_FRAME_IN);
	symCreateIntMacro("HTG_BORDER_FRAME_OUT",HTG_BORDER_FRAME_OUT);
	symCreateIntMacro("HTG_BORDER_BOX",HTG_BORDER_BOX);

	symCreateIntMacro("HTG_ALIGN_LEFT",HTG_ALIGN_LEFT);
	symCreateIntMacro("HTG_ALIGN_RIGHT",HTG_ALIGN_RIGHT);
	symCreateIntMacro("HTG_ALIGN_CENTER",HTG_ALIGN_CENTER);

	symCreateIntMacro("HTG_X_AXIS",HTG_X_AXIS);
	symCreateIntMacro("HTG_Y_AXIS",HTG_Y_AXIS);

	symCreateIntMacro("HTG_FONT_TINY",HTG_FONT_TINY);
	symCreateIntMacro("HTG_FONT_SMALL",HTG_FONT_SMALL);
	symCreateIntMacro("HTG_FONT_MEDIUM_BOLD",HTG_FONT_MEDIUM_BOLD);
	symCreateIntMacro("HTG_FONT_LARGE",HTG_FONT_LARGE);
	symCreateIntMacro("HTG_FONT_GIANT",HTG_FONT_GIANT);

	symCreateIntMacro("HTG_DASH_MARKS",HTG_DASH_MARKS);

	symCreateIntMacro("HTG_SIGNED",HTG_SIGNED);
	symCreateIntMacro("HTG_UNSIGNED",HTG_UNSIGNED);

	symCreateCharMacro("HTG_STDOUT","");

	symCreateIntMacro("HTG_LINE_GRAPH",HTG_LINE_GRAPH);
	symCreateIntMacro("HTG_BAR_GRAPH",HTG_BAR_GRAPH);
	symCreateIntMacro("HTG_FLAT_TOP_GRAPH",HTG_FLAT_TOP_GRAPH);

#ifndef STATIC_LIB
	if (modLoadModuleFunctions(modPtr,graph_efuncts) < 0)
	{
		return(-1);
	}
#endif
	return(0);
}




/*************************************************************************
**									**
**		     	    Utility Routines				**
**									**
*************************************************************************/


static int power(p)
	int	p;
{
	int	val = 1;
	while (p)
	{
		val *= 10;
		p--;
	}
	return(val);
}


static void bevelArea(img,x1,y1,x2,y2, red, green, blue, borderWidth, direction)
	gdImage	*img;
	int	x1,y1,
		x2,y2,
		red, green, blue,
		borderWidth,
		direction;
{
	int	bevDark,
		bevLight,
		col1, col2;
	gdPoint	points[8];



	bevDark = gdImageColorAllocate(img,
		red-25 < 0? 0: red-25,
		green-25 < 0? 0: green-25,
		blue-25 < 0? 0: blue-25);
	bevLight = gdImageColorAllocate(img,
		red+25 > 255? 255: red+25,
		green+25 > 255? 255: green+25,
		blue+25 > 255? 255: blue+25);
	if (borderWidth < 1)
		borderWidth = 3;

	if (direction == HTG_BORDER_BEVEL_IN)
	{
		col1 = bevDark;
		col2 = bevLight;
	}
	else
	{
		col1 = bevLight;
		col2 = bevDark;
	}

	/* Top half of the bevel */
	points[0].x = x1;
	points[0].y = y1;
	points[1].x = x2 - 1;
	points[1].y = y1;
	points[2].x = x2 - 1 - borderWidth;
	points[2].y = y1 + borderWidth;
	points[3].x = x1 + borderWidth;
	points[3].y = y1 + borderWidth;
	points[4].x = x1 + borderWidth;
	points[4].y = y2 -1 -borderWidth;
	points[5].x = x1;
	points[5].y = y2 -1;
	gdImageFilledPolygon(img, points, 6, col1);

	/* Bottom half of the bevel */
	points[0].x = x1;
	points[0].y = y2 -1;
	points[1].x = x2 - 1;
	points[1].y = y2 -1;
	points[2].x = x2 - 1;
	points[2].y = y1;
	points[3].x = x2 - 1 - borderWidth;
	points[3].y = y1 + borderWidth;
	points[4].x = x2 -1 - borderWidth;
	points[4].y = y2 -1 -borderWidth;
	points[5].x = x1 + borderWidth;
	points[5].y = y2 -1 -borderWidth;
	gdImageFilledPolygon(img, points, 6, col2);
}



static void frameArea(img,x1,y1,x2,y2, red, green, blue, width, direction)
	gdImage	*img;
	int	x1,y1,
		x2,y2,
		red, green, blue,
		width,
		direction;
{
	int	colDark,
		colLight,
		colNorm,
		count,
		col1, col2;

	colNorm = gdImageColorAllocate(img,red,green,blue);
	colDark = gdImageColorAllocate(img,
		red-25 < 0? 0: red-25,
		green-25 < 0? 0: green-25,
		blue-25 < 0? 0: blue-25);
	colLight = gdImageColorAllocate(img,
		red+25 > 255? 255: red+25,
		green+25 > 255? 255: green+25,
		blue+25 > 255? 255: blue+25);

	if (direction == HTG_BORDER_FRAME_OUT)
	{
		col1 = colDark;
		col2 = colLight;
	}
	else
	{
		col1 = colLight;
		col2 = colDark;
	}

	gdImageLine(img, x1, y1, x2, y1, col2);
	count = 1;
	while(count <= width-2)
	{
		gdImageLine(img, x1, y1+count, x2, y1+count, colNorm);
		count++;
	}
	gdImageLine(img, x1, y1+(width-1), x2, y1+width-1, col1);

	gdImageLine(img, x1, y2-(width-1), x2, y2-(width-1), col2);
	count = 1;
	while(count <= width-2)
	{
		gdImageLine(img, x1, y2-count, x2, y2-count, colNorm);
		count++;
	}
	gdImageLine(img, x1, y2, x2, y2, col1);

	gdImageLine(img, x1, y1, x1, y2, col2);
	count = 1;
	while(count <= width-2)
	{
		gdImageLine(img, x1+count,y1+count,x1+count,y2-count,colNorm);
		count++;
	}
	gdImageLine(img, x1+(width-1), y1+(width-1), x1+(width-1), 
		y2-(width-1), col1);

	gdImageLine(img, x2-(width-1), y1+(width-1), x2-(width-1),
		y2-(width-1), col2);
	count = 1;
	while(count <= width-2)
	{
		gdImageLine(img, x2-count,y1+count,x2-count,y2-count,colNorm);
		count++;
	}
	gdImageLine(img, x2, y1, x2, y2, col1);
}





static void boxArea(img,x1,y1,x2,y2,width, col)
	gdImage	*img;
	int	x1,y1,
		x2,y2,
		width,
		col;
{
	gdImageFilledRectangle(img, x1, y1, x2, y1+width, col);
	gdImageFilledRectangle(img, x1, y1, x1+width, y2, col); 
	gdImageFilledRectangle(img, x2-width, y1, x2, y2, col);
	gdImageFilledRectangle(img, x1, y2-width, x2, y2, col);
}

	

static void drawTitle(graph)
	htgGraph *graph;
{
	htgTitle *title;
	int	tX1, tY1, tX2, tY2,
		textX, textY,
		red, green, blue,
		borderWidth;
	gdImage	*img;
	gdFontPtr font;

	title = graph->title;
	if (!title)
		return;

	borderWidth = title->borderWidth;
	red = graph->bgRed;
	green = graph->bgGreen;
	blue = graph->bgBlue;
	img = graph->img;

	/*
	** Grab the font info
	*/
	switch(title->font)
	{
		case HTG_FONT_TINY:
			font = gdFontTiny;
			break;
		case HTG_FONT_SMALL:
			font = gdFontSmall;
			break;
		case HTG_FONT_MEDIUM_BOLD:
			font = gdFontMediumBold;
			break;
		case HTG_FONT_LARGE:
			font = gdFontLarge;
			break;
		case HTG_FONT_GIANT:
			font = gdFontGiant;
			break;
		default:
			runError("Unknown or missing font!");
	}

	/*
	** Draw the title border if required
	*/
	tX1 = graph->cX;
	tX2 = tX1 + graph->cWidth;
	tY1 = graph->cX;
	tY2 = tY1 + (2 * borderWidth) + font->h + 4;

	switch(title->borderType)
	{
		case HTG_BORDER_NONE:
			borderWidth = 0;
			break;

		case HTG_BORDER_BEVEL_OUT:
			tX1 +=1;
			tY1 +=1;
			tX2 -=1;
			tY2 -=1;
			bevelArea(img,tX1, tY1, tX2, tY2,
				red, green, blue, borderWidth,
				HTG_BORDER_BEVEL_OUT);
			break;

		case HTG_BORDER_BEVEL_IN:
			tX1 +=1;
			tY1 +=1;
			tX2 -=1;
			tY2 -=1;
			bevelArea(img,tX1, tY1, tX2, tY2,
				red, green, blue, borderWidth,
				HTG_BORDER_BEVEL_IN);
			break;

		case HTG_BORDER_FRAME_IN:
			if (borderWidth < 3)
				borderWidth=3;
			frameArea(img,tX1, tY1, tX2, tY2,
				red, green, blue, borderWidth,
				HTG_BORDER_FRAME_IN);
			break;

		case HTG_BORDER_FRAME_OUT:
			if (borderWidth < 3)
				borderWidth=3;
			frameArea(img,tX1, tY1, tX2, tY2,
				red, green, blue, borderWidth,
				HTG_BORDER_FRAME_OUT);
			break;

		case HTG_BORDER_BOX:
			if (borderWidth < 1)
				borderWidth = 1;
			boxArea(img,tX1,tY1,tX2,tY2,borderWidth,graph->fgCol);
			break;
	}


	/*
	** Draw the text
	*/
	switch(title->align)
	{
		case HTG_ALIGN_CENTER:
			textX = tX1 + ((tX2-tX1)/2) - 
				(strlen(title->title) * font->w / 2);
			break;

		case HTG_ALIGN_LEFT:
			textX = tX1 + 2;
			break;

		case HTG_ALIGN_RIGHT:
			textX = tX2 - (strlen(title->title) * font->w) -2;
			break;
	}
	textY = tY1 + ((tY2-tY1)/2) - font->h / 2;
	gdImageString(img, font, textX, textY, title->title, 
		graph->fgCol);


	graph->cY = tY2 + 15;
	graph->cHeight -= (tY2-tY1) +15;
}



static int checkFlag(graph, flag)
	htgGraph *graph;
	int	flag;
{
	return(graph->flags & flag);
}


/*************************************************************************
**									**
**		     Graph Creation & Output Routines			**
**									**
*************************************************************************/



void htgCreateGraph(params)
	plist_t	*params;
{
	int	width,
		height,
		bgRed, bgBlue, bgGreen,
		fgRed, fgBlue, fgGreen,
		borderType,
		borderWidth,
		object,
		fgCol,
		precision,
		sign;
	gdImage	*img;
	htgGraph *graph;

	/*
	** Grab the params
	*/
	width = (int) * (int *)params->sym->val;
	height = (int) * (int *)params->next->sym->val;
	bgRed = (int) * (int *)params->next->next->sym->val;
	bgGreen = (int) * (int *)params->next->next->next->sym->val;
	bgBlue = (int) * (int *)params->next->next->next->next->sym->val;
	fgRed = (int) * (int *)params->next->next->next->next->next->sym->val;
	fgGreen = (int) * (int *)
		params->next->next->next->next->next->next->sym->val;
	fgBlue = (int) * (int *)
		params->next->next->next->next->next->next->next->sym->val;
	borderType = (int) * (int *)
	 params->next->next->next->next->next->next->next->next->sym->val;
	borderWidth = (int) * (int *)
	 params->next->next->next->next->next->next->next->next->next->sym->val;
	precision = (int) * (int *)
	 params->next->next->next->next->next->next->next->next->next->next->sym->val;
	sign = (int) * (int *)
	 params->next->next->next->next->next->next->next->next->next->next->next->sym->val;


	/*
	** Create the basic canvas
	*/
	graph = (htgGraph *)malloc(sizeof(htgGraph));
	if (!graph)
	{
                setError("Out of memory");
                externReturn = createIntSymbol(-1);
                return;
	}
	bzero(graph,sizeof(htgGraph));
	img = graph->img = gdImageCreate(width, height);
	if (!graph->img)
	{
                setError("Couldn't create image");
                externReturn = createIntSymbol(-1);
                return;
	}
	graph->bgCol = gdImageColorAllocate(graph->img,bgRed, bgGreen, bgBlue);
	graph->fgCol = gdImageColorAllocate(graph->img,fgRed, fgGreen, fgBlue);
	fgCol = graph->fgCol;
	graph->width = width;
	graph->height = height;
	graph->bgRed = bgRed;
	graph->bgGreen = bgGreen;
	graph->bgBlue = bgBlue;
	graph->xTicks = graph->yTicks = NULL;
	graph->flags = 0;
	graph->precision = precision;
	graph->sign = sign;

	/*
	** Setup the border
	*/
	switch (borderType)
	{
		case HTG_BORDER_NONE:
			borderWidth = 0;
			break;

		case HTG_BORDER_BEVEL_OUT:
			bevelArea(img,0,0,width-1,height-1,
				bgRed, bgGreen, bgBlue, borderWidth,
				HTG_BORDER_BEVEL_OUT);
			break;

		case HTG_BORDER_BEVEL_IN:
			bevelArea(img,0,0,width-1,height-1, 
				bgRed, bgGreen, bgBlue, borderWidth,
				HTG_BORDER_BEVEL_IN);
			break;

		case HTG_BORDER_FRAME_IN:
			if (borderWidth < 3)
				borderWidth=3;
			frameArea(img,2,2,width-3,height-3, 
				bgRed, bgGreen, bgBlue, borderWidth,
				HTG_BORDER_FRAME_IN);
			break;

		case HTG_BORDER_FRAME_OUT:
			if (borderWidth < 3)
				borderWidth=3;
			frameArea(img,2,2,width-3,height-3, 
				bgRed, bgGreen, bgBlue, borderWidth,
				HTG_BORDER_FRAME_OUT);
			break;

		case HTG_BORDER_BOX:
			if (borderWidth < 1)
				borderWidth = 1;
			boxArea(img,2,2,width-3,height-3,borderWidth,
				graph->fgCol);
			break;
			
	}


	/*
	** Return the graph object
	*/
	graph->cX = borderWidth + 5;
	graph->cY = borderWidth + 5;
	graph->cWidth = width - (2 * (borderWidth + 5));
	graph->cHeight = height - (2 * (borderWidth + 5));
	graph->data = NULL;
	graph->title = NULL;
	graph->minValue = -INT_MAX;
	graph->maxValue = INT_MAX;
	object = storeObject((char *)graph);
	externReturn = createIntSymbol(object);
}




void htgOutputGraph(params)
	plist_t	*params;
{
	int	object;
	char	*path;
	htgGraph *graph;
	FILE	*out;

	checkContentType();
	object = (int) * (int*)params->sym->val;
	path = params->next->sym->val;
	graph = (htgGraph *)fetchObject(object);
	if (!graph)
	{
		setError("Invalid graph handle");
                externReturn = createIntSymbol(-1);
                return;
	}

	out = stdout;
	if (path)
	{
		if (*path)
		{
			out = fopen(path,"w");
		}
	}
	gdImageGif(graph->img, out);
	if (out != stdout)
	{
		fclose(out);
	}
}

 
void htgDestroyGraph(params)
	plist_t	*params;
{
	int	object;
	htgGraph *graph;

	object = (int) * (int*)params->sym->val;
	graph = (htgGraph *)fetchObject(object);
	if (!graph)
	{
		setError("Invalid graph handle");
                externReturn = createIntSymbol(-1);
                return;
	}

	gdImageDestroy(graph->img);
	externReturn = createIntSymbol(0);
}


void htgSetGraphTitle(params)
	plist_t	*params;
{
	int	object;
	htgGraph *graph;
	char	*title;
	int	border,
		borderWidth,
		align,
		font;

	object = (int) * (int*)params->sym->val;
	title = params->next->sym->val;
	border = (int)*(int*)params->next->next->sym->val;
	borderWidth = (int)*(int*)params->next->next->next->sym->val;
	align = (int)*(int*)params->next->next->next->next->sym->val;
	font = (int)*(int*)params->next->next->next->next->next->sym->val;

	graph = (htgGraph *)fetchObject(object);
	if (!graph)
	{
		setError("Invalid graph handle");
                externReturn = createIntSymbol(-1);
                return;
	}
	if (graph->title)
	{
		setError("Graph already has a title");
                externReturn = createIntSymbol(-1);
                return;
	}
	graph->title = (htgTitle *)malloc(sizeof(htgTitle));
	graph->title->title = (char *)strdup(title);
	graph->title->borderType = border;
	graph->title->borderWidth = borderWidth;
	graph->title->align = align;
	graph->title->font = font;
	externReturn = createIntSymbol(0);
}



void htgSetGraphRange(params)
	plist_t	*params;
{
	int	object,
		min, max;
	htgGraph *graph;

	object = (int) * (int*)params->sym->val;
	min = (int) * (int *)params->next->sym->val;
	max = (int) * (int *)params->next->next->sym->val;

	graph = (htgGraph *)fetchObject(object);
	if (!graph)
	{
		setError("Invalid graph handle");
                externReturn = createIntSymbol(-1);
                return;
	}
	graph->minValue = min;
	graph->maxValue = max;
	externReturn = createIntSymbol(0);
}


void htgSetFlag(params)
	plist_t	*params;
{
	int	object,
		flag;
	htgGraph *graph;

	object = (int) * (int*)params->sym->val;
	flag = (int) * (int *)params->next->sym->val;

	graph = (htgGraph *)fetchObject(object);
	if (!graph)
	{
		setError("Invalid graph handle");
                externReturn = createIntSymbol(-1);
                return;
	}
	graph->flags = graph->flags | flag;
	externReturn = createIntSymbol(0);
}

void htgCreateLegend(params)
	plist_t	*params;
{
	int	object,
		borderType,
		borderWidth,
		fontName;
	htgGraph *graph;
	htgData *cur;
        gdFontPtr font;
	gdImage *img;
	int	bbx1,bby1,bbx2,bby2,
		barX, barWidth, barOff,
		textX,
		curY,
		maxLen,
		count,
		col;

	object = (int) * (int*)params->sym->val;
	borderType = (int)*(int*)params->next->sym->val;
	borderWidth = (int)*(int*)params->next->next->sym->val;
	fontName = (int)*(int*)params->next->next->next->sym->val;

	graph = (htgGraph *)fetchObject(object);
	if (!graph)
	{
		setError("Invalid graph handle");
                externReturn = createIntSymbol(-1);
                return;
	}
	img = graph->img;

	/*
	** Grab the font
	*/
	switch(fontName)
	{
		case HTG_FONT_TINY:	
			font = gdFontTiny;
			break;
		case HTG_FONT_SMALL:	
			font = gdFontSmall;
			break;
		case HTG_FONT_MEDIUM_BOLD:	
			font = gdFontMediumBold;
			break;
		case HTG_FONT_LARGE:	
			font = gdFontLarge;
			break;
		case HTG_FONT_GIANT:	
			font = gdFontGiant;
			break;
	}


	/*
	** Work out and draw the bounding box
	*/
	bbx1 = graph->cX;
	bbx2 = graph->cX + graph->cWidth;
	bby2 = graph->cY + graph->cHeight;
	count = 0;
	maxLen = 0;
	cur = graph->data;
	while(cur)
	{
		if (strlen(cur->name) == 0)
		{
			cur=cur->next;
			continue;
		}
		count++;
		if (strlen(cur->name) > maxLen)
			maxLen = strlen(cur->name);
		cur = cur->next;
	}
	bby1=graph->cY + graph->cHeight - (count * (font->h + 2)) 
		- borderWidth - 5;

	switch(borderType)
	{
		case HTG_BORDER_NONE:
			borderWidth = 0;
			break;

		case HTG_BORDER_BEVEL_OUT:
		case HTG_BORDER_BEVEL_IN:
			bevelArea(img,bbx1, bby1, bbx2, bby2,
				graph->bgRed, graph->bgGreen, graph->bgBlue, 
				borderWidth, borderType);
			break;

		case HTG_BORDER_FRAME_OUT:
		case HTG_BORDER_FRAME_IN:
			if (borderWidth < 3)
				borderWidth=3;
			frameArea(img,bbx1, bby1, bbx2, bby2,
				graph->bgRed, graph->bgGreen, graph->bgBlue, 
				borderWidth, borderType);
			break;

		case HTG_BORDER_BOX:
			if (borderWidth < 1)
				borderWidth = 1;
			boxArea(img,bbx1,bby1,bbx2,bby2,borderWidth,
				graph->fgCol);
			break;
	}

	/*
	** Fill in the details
	*/
	gdImageString(img, gdFontMediumBold, bbx1 + 5, bby1 + 3, "Legend", 
		graph->fgCol);

	textX = bbx2 -3 - (font->w * maxLen);
	barWidth = ((textX - 10) - (bbx1 + 30)) * .5;
	barX = textX - (1.1 * barWidth);
	barOff = font->h / 2;
	curY = bby1 + 5;
	cur = graph->data;
	while(cur)
	{
		if (strlen(cur->name) == 0)
		{
			cur = cur->next;
			continue;
		}
		gdImageString(img, font, textX, curY, cur->name,graph->fgCol);
		col = gdImageColorAllocate(img,cur->red, cur->green,cur->blue);
		gdImageLine(img, barX, curY + barOff - 1, barX + barWidth, 
			curY + barOff - 1, col);
		gdImageLine(img, barX, curY + barOff, barX + barWidth, 
			curY + barOff, col);
		curY = curY + font->h + 2;
		cur = cur->next;
	}

	graph->cHeight -= bby2 - bby1 + 15;
}

/*************************************************************************
**									**
**		     	Data Management Routines			**
**									**
*************************************************************************/


static void addDataSeries(graph,name,array,red,green,blue,graphType,width)
	htgGraph *graph;
	char	*name;
	sym_t	*array;
	int	red, green, blue,
		graphType,
		width;
{
	htgData *series,
		*tail;
	sym_t	 *cur;
	int	count;


	/*
	** Setup the series structure
	*/
	series = (htgData *)malloc(sizeof(htgData));
	if (!series)
	{
		setError("Out of memory");
                externReturn = createIntSymbol(-1);
                return;
	}
	series->numValues = symGetNumArrayElements(array);
	series->red = red;
	series->green = green;
	series->blue = blue;
	series->data = (double *)malloc(series->numValues * sizeof(double));
	series->name = (char *)strdup(name);
	series->graphType = graphType;
	series->width = width;
	series->next = NULL;

	tail = graph->data;
	if (!tail)
	{
		graph->data = series;
	}
	else
	{
		while(tail->next)
		{
			tail = tail->next;
		}
		tail->next = series;
	}


	/*
	** Load the data
	*/
	tail = NULL;
	count = 0;
	while(count < series->numValues)
	{
		cur = symGetArrayElement(array,count);
		series->data[count] = atof(cur->val);
		count++;
	}

	externReturn = createIntSymbol(0);
}


void htgAddLineGraphData(params)
	plist_t	*params;
{
	int	object,
		red, green, blue;
	htgGraph *graph;
	sym_t	*array;
	char	*name;

	/*
	** Process the params
	*/
	object = (int) * (int*)params->sym->val;
	name = params->next->sym->val;
	array = params->next->next->sym;
	red = (int) * (int*)params->next->next->next->sym->val;
	green = (int) * (int*)params->next->next->next->next->sym->val;
	blue = (int) * (int*)params->next->next->next->next->next->sym->val;

	graph = (htgGraph *)fetchObject(object);
	if (!graph)
	{
		setError("Invalid graph handle");
                externReturn = createIntSymbol(-1);
                return;
	}
	addDataSeries(graph,name,array,red,green,blue,HTG_LINE_GRAPH,0);
}


void htgAddBarGraphData(params)
	plist_t	*params;
{
	int	object,
		red, green, blue,
		width;
	htgGraph *graph;
	sym_t	*array;
	char	*name;

	/*
	** Process the params
	*/
	object = (int) * (int*)params->sym->val;
	name = params->next->sym->val;
	array = params->next->next->sym;
	red = (int) * (int*)params->next->next->next->sym->val;
	green = (int) * (int*)params->next->next->next->next->sym->val;
	blue = (int) * (int*)params->next->next->next->next->next->sym->val;
	width = (int) * (int*)params->next->next->next->next->next->next->sym->val;

	graph = (htgGraph *)fetchObject(object);
	if (!graph)
	{
		setError("Invalid graph handle");
                externReturn = createIntSymbol(-1);
                return;
	}
	addDataSeries(graph,name,array,red,green,blue,
		HTG_BAR_GRAPH,width);
}




void htgAddFlatTopGraphData(params)
	plist_t	*params;
{
	int	object,
		red, green, blue,
		width;
	htgGraph *graph;
	sym_t	*array;
	char	*name;

	/*
	** Process the params
	*/
	object = (int) * (int*)params->sym->val;
	name = params->next->sym->val;
	array = params->next->next->sym;
	red = (int) * (int*)params->next->next->next->sym->val;
	green = (int) * (int*)params->next->next->next->next->sym->val;
	blue = (int) * (int*)params->next->next->next->next->next->sym->val;
	width = (int) * (int*)params->next->next->next->next->next->next->sym->val;

	graph = (htgGraph *)fetchObject(object);
	if (!graph)
	{
		setError("Invalid graph handle");
                externReturn = createIntSymbol(-1);
                return;
	}
	addDataSeries(graph,name,array,red,green,blue,
		HTG_FLAT_TOP_GRAPH,width);
}





void htgAddTicks(params)
	plist_t	*params;
{
	int	object,
		count,
		axis;
	htgGraph *graph;
	htgTick *ticks;
	sym_t	*array,
		*cur;

	/*
	** Process the params
	*/
	object = (int) * (int*)params->sym->val;
	array = params->next->sym;
	axis = (int) * (int*)params->next->next->sym->val;

	graph = (htgGraph *)fetchObject(object);
	if (!graph)
	{
		setError("Invalid graph handle");
                externReturn = createIntSymbol(-1);
                return;
	}

	/*
	** Setup the tick structure
	*/
	if ((axis == HTG_X_AXIS && graph->xTicks) || 
	    (axis == HTG_Y_AXIS && graph->yTicks))
	{
		setError("Ticks already set for this axis");
                externReturn = createIntSymbol(-1);
                return;
	}
	ticks = (htgTick *)malloc(sizeof(htgTick));
	if (!ticks)
	{
		setError("Out of memory");
                externReturn = createIntSymbol(-1);
                return;
	}
	ticks->numTicks = symGetNumArrayElements(array);
	ticks->ticks = (char **)malloc(ticks->numTicks * sizeof(char *));


	/*
	** Load the ticks
	*/
	count = 0;
	while(count < ticks->numTicks)
	{
		cur = symGetArrayElement(array,count);
		if (cur->val)
		{
			ticks->ticks[count] = (char *)strdup(cur->val);
		}
		else
		{
			ticks->ticks[count] = (char *)strdup("");
		}
		count++;
	}
	if (axis == HTG_X_AXIS)
		graph->xTicks = ticks;
	else
		graph->yTicks = ticks;
	externReturn = createIntSymbol(0);
}




/*************************************************************************
**									**
**		     	    Graphing  Routines				**
**									**
*************************************************************************/

#define	 NUM_Y_TICKS 16	

static void drawGraphTicks(graph, minY, maxY, sign, graphType)
	htgGraph *graph;
	int	minY, maxY,
		sign,
		graphType;
{
	htgTick	*xTicks,
		*yTicks;
	int	x1,y1,x2,y2,
		col,
		axisWidth, 
		curX, curY, 
		textX, textY, textWidth,
		count,
		numYTicks;
	char	textBuf[80],
		format[10];
	gdImage *img;
        gdFontPtr font;
	double	stepVal,
		xInc, yInc,
		curVal;
	u_int	stepUVal,
		curUVal;

	xTicks = graph->xTicks;
	yTicks = graph->yTicks;
	img = graph->img;
	col = graph->fgCol;


	/*
	** Determine how wide the Y axis tick text is going to be
	*/
	if ((double)(graph->maxValue - graph->minValue) > 
	    (int)(graph->maxValue - graph->minValue))
	{
		count = (int)(graph->maxValue - graph->minValue)+1;
	}
	else
	{
		count = (int)(graph->maxValue - graph->minValue);
	}
	if (count < NUM_Y_TICKS)
	{
		if (count < 8)
			count = 8;
		numYTicks = count;
	}
	else
	{
		numYTicks = NUM_Y_TICKS;
	}
	font = gdFontSmall;
	if (sign == HTG_SIGNED)
		sprintf(textBuf,"%d",(int)graph->maxValue);
	else
		sprintf(textBuf,"%u",(uint)graph->maxValue);
	stepVal = ((float)(graph->maxValue - graph->minValue) / 
			(float)numYTicks);
	if (stepVal < 1 && graph->precision == 0)
		graph->precision = 2;
	if (stepVal < 2 && graph->precision == 0)
		graph->precision = 1;
	if (graph->precision == 0)
  		textWidth = strlen(textBuf) * font->w + 6;
	else
  		textWidth = (strlen(textBuf)+graph->precision+1) * font->w + 6;
	
	/*
	** Draw in the axis
	*/
	axisWidth = graph->cWidth - textWidth - 5;
	x1 = graph->cX + textWidth;
	y1 = graph->cY + graph->cHeight;
	x2 = graph->cX - 5 + graph->cWidth;
	y2 = graph->cY + graph->cHeight;
	gdImageLine(img,x1,y1,x2,y2,col);
	gdImageLine(img,x1,y1+1,x2,y2+1,col);

	x1 = graph->cX + textWidth;
	y1 = graph->cY + graph->cHeight;
	x2 = graph->cX + textWidth;
	y2 = graph->cY - 8;
	gdImageLine(img,x1,y1,x2,y2,col);
	gdImageLine(img,x1+1,y1,x2+1,y2,col);


	/*
	** Draw in the X tick marks
	*/
	x1 = graph->cX + textWidth;
	y1 = graph->cY + graph->cHeight;
	if (xTicks)
	{
		xInc = (double)axisWidth / (double)xTicks->numTicks;
		count = 0;
		while(count < xTicks->numTicks)
		{
			curX = x1 + (xInc * count) + (xInc / 2);
			gdImageLine(img,curX,y1-2, curX,y1+2, col);
			textX = curX - (strlen(xTicks->ticks[count]) *
				font->w / 2); 
			textY = y1 + 2;
			gdImageString(img, font, textX, textY, 
				xTicks->ticks[count], col);
			count++;
		}
	}

	/*
	** Draw in the Y tick marks
	*/

	x1 = graph->cX + textWidth;
	y1 = graph->cY + graph->cHeight - 1;
	yInc = (float)graph->cHeight / (float)numYTicks;

#ifdef DEBUG
printf("Height = %d  yInc = %f\n", graph->cHeight, yInc);
#endif


	curY = y1 ;
	count = 0;
	stepVal = ((double)(graph->maxValue - graph->minValue) / 
			(double)numYTicks);

	while(count <= numYTicks)
	{
		curY = y1 - ((float)count * yInc);
		curVal = graph->minValue + ((double)count * stepVal);

		sprintf(format,"%%.%df",graph->precision);
		sprintf(textBuf,format,curVal);
#ifdef DEBUG
printf("Tick Y = %d for '%s'\n",curY,textBuf);
#endif

		gdImageLine(img,x1-2,curY, x1+2,curY, col);
		textX = x1 - (strlen(textBuf) * font->w) - 3;
		textY = curY - font->h + 5;
		gdImageString(img, font, textX, textY, textBuf,col);
		if (checkFlag(graph, HTG_DASH_MARKS))
		{
			gdImageDashedLine(img, x1 + 5, curY, x1 + axisWidth,
				curY,col);
		}
		count++;
	}


	/*
	** Reset the canvas
	*/
	graph->cX += textWidth + 1;
	graph->cWidth = axisWidth;
}





static void drawFlatTopGraph(graph, img, series, x1, y1, x2, y2, col)
	htgGraph *graph;
	gdImage	*img;
	htgData	*series;
	int	x1, y1,
		x2, y2,
		col;
{
	int	curX,
		segWidth,
		width;

	segWidth = x2 - x1;
	/*gdImageLine(img,x1,y2-1,x2,y2-1,col);*/
	gdImageLine(img,x1,y2,x2,y2,col);
	gdImageLine(img,x1,y2+1,x2,y2+1,col);
}


static void drawBarGraph(graph, img, series, x1, y1, x2, y2, col)
	htgGraph *graph;
	gdImage	*img;
	htgData	*series;
	int	x1, y1,
		x2, y2,
		col;
{
	int	curX,
		segWidth,
		width;

	segWidth = x2 - x1;
	width = (int)((double)segWidth * ((double)series->width / 100));

	x1 = x1 + ((segWidth - width) / 2);
	x2 = x2 - ((segWidth - width) / 2);
	if (width >= 5)
	{
		gdImageRectangle(img, x1,y1,x2,y2,graph->fgCol);
	}
	curX = x1+1;
	while(curX < x2)
	{
		gdImageLine(img,curX,y1-1,curX,y2+1,col);
		curX++;
	}
}



static void drawLineGraph(img, curX, curY, prevX, prevY, col)
	gdImage	*img;
	int	curX, curY,
		prevX, prevY,
		col;
{
	int	absX, absY;

	gdImageLine(img, prevX, prevY, curX,  curY, col);

	/*
	** Ensure the line is 2 pixels wide
	*/
	absX = curX - prevX;
	if (absX < 0)
		absX = 0 - absX;
	absY = curY - prevY;
	if (absY < 0)
		absY = 0 - absY;
	if (absX < absY)
	{
		gdImageLine(img, prevX+1, prevY, curX+1, 
			curY, col);
	}
	else
	{
		gdImageLine(img, prevX, prevY+1, curX, 
			curY+1, col);
	}
}



void htgPlotGraph(params)
	plist_t	*params;
{
	int	object;
	htgGraph *graph;
	htgData *cur;

	int	numX,
		count,
		curX, curY,
		prevX, prevY,
		baseX, baseY,
		col;
	double	minY, maxY;
	int	xRange,
		yRange;
	float	yInc,
		xInc;
	gdImage	*img;

	/*
	** Process the params
	*/
	object = (int) * (int*)params->sym->val;
	graph = (htgGraph *)fetchObject(object);
	if (!graph)
	{
		setError("Invalid graph handle");
                externReturn = createIntSymbol(-1);
                return;
	}
	img = graph->img;

	drawTitle(graph);

	/*
	** Check out the data
	*/
	numX = 0;
	if (graph->sign == HTG_SIGNED)
	{
		minY = INT_MAX;
		maxY = 0 - INT_MAX;
	}
	else
	{
		minY = UINT_MAX;
		maxY = 0;
	}
	cur = graph->data;
	while(cur)
	{
		if (cur->numValues > numX)
			numX = cur->numValues;
		count = 0;
		while(count < cur->numValues)
		{
			if ((double)cur->data[count] > maxY)
				maxY = (double)cur->data[count];
			if ((double)cur->data[count] < minY)
				minY = (double)cur->data[count];
			count ++;
		}
		cur = cur->next;
	}

	if (graph->sign == HTG_UNSIGNED)
	{
		if (minY < 0 )
		{
			printf("Error : Negative values in unsigned graph!\n");
			exit(1);
		}
	}
	if ((int)maxY != maxY)
	{
		maxY = (int)maxY + 1;
	}

	if (graph->minValue != -INT_MAX)
		minY = (double)graph->minValue;
	else
		graph->minValue = minY;
	if (graph->maxValue != INT_MAX)
		maxY = graph->maxValue;
	else
		graph->maxValue = maxY;

	xRange = numX - 1;
	if (xRange == -1)
		xRange = 0;
	if (graph->sign == HTG_SIGNED)
		yRange = maxY - minY;
	else
		yRange = (unsigned)maxY - (unsigned)minY;
	if (yRange == 0)
		yRange = 1;

	drawGraphTicks(graph, minY, maxY, graph->sign, HTG_LINE_GRAPH);

	if (graph->sign == HTG_SIGNED)
	{
		xInc = ((float)graph->cWidth / (float)(xRange + 1));
		yInc = (float)graph->cHeight / (float)yRange;
	}
	else
	{
		xInc = ((float)graph->cWidth / (float)(xRange + 1));
		yInc = ((float)graph->cHeight / (float)((unsigned)yRange));
	}

	/*
	** Plot the data
	*/
	baseX = graph->cX + (xInc / 2);
	baseY = graph->cY + graph->cHeight - 1;

#ifdef DEBUG
printf("Height = %d  Base Y = %d  yInc = %f\n",graph->cHeight, baseY, yInc);
#endif

	cur = graph->data;
	while(cur)
	{
		col = gdImageColorAllocate(img, cur->red, cur->green,
			cur->blue);
		count = prevX = prevY = 0;
		while(count < cur->numValues)
		{
			curX = baseX + (count * xInc);
			if (graph->sign == HTG_SIGNED)
			{
				curY = baseY - (
					(cur->data[count] - (double)minY) *
					(double)yInc);
			}
			else
			{
				curY = baseY - (
					(cur->data[count] 
					- (double)(unsigned)minY) * 
					(double)yInc);
			}
#ifdef DEBUG
printf("Data Y = %d for '%d'\n",curY,cur->data[count]);
#endif
			if (cur->graphType == HTG_LINE_GRAPH)
			{
				if (count == 0)
				{
					prevX = curX;
					prevY = curY;
					count++;
					continue;
				}
				drawLineGraph(img,curX,curY,prevX,prevY,col);
			}
			if (cur->graphType == HTG_BAR_GRAPH)
			{
				if (count == 0)
					prevX = baseX - (xInc / 2);
				curX += (xInc / 2);
				drawBarGraph(graph,img,cur,prevX,baseY,curX,
					curY,col);
			}
			if (cur->graphType == HTG_FLAT_TOP_GRAPH)
			{
				if (count == 0)
					prevX = baseX - (xInc / 2);
				curX += (xInc / 2);
				drawFlatTopGraph(graph,img,cur,prevX,baseY,curX,
					curY,col);
			}
			prevX = curX;
			prevY = curY;
			count++;
		}
		cur = cur->next;
	}
	externReturn = createIntSymbol(0);
}
