/*
**	mod_graph.h - Lite / W3-mSQL module for the graphing library
**
**
** Copyright (c) 1997  Hughes Technologies Pty Ltd.
**
** Permission to use, copy, and distribute for non-commercial purposes,
** is hereby granted without fee, providing that the above copyright
** notice appear in all copies and that both the copyright notice and this
** permission notice appear in supporting documentation.
**
** The software may be modified for your own purposes, but modified versions
** may not be distributed.
**
** This software is provided "as is" without any expressed or implied warranty.
**
*/




/*
** External function prototypes
*/


/*
** Note : As this is a dynamic loading module (rather than a static
** linked module) we don't need the function pointer field to be
** completed.  It will be filled at run-time as the module is loaded
** into Lite's process address space.
*/

/*
** Process related module function definitions
**
** This table maps from internal Lite function names to the C code.
** The format is :-
**
**	{LiteName, CFunctPtr, CFunctName, NumArgs, { Arg1Type, Arg2Type.., 0}}
**
**	Setting NumArgs to -1 indicates varargs
*/


efunct_t graph_efuncts[] = {

	/* Graph creation & output routines */
        { "htgCreateGraph", NULL, "_htgCreateGraph", 10, 
		{P_INT, P_INT, P_INT, P_INT, P_INT, P_INT, P_INT,
		 P_INT, P_INT, P_INT, 0} },
        { "htgOutputGraph", NULL, "_htgOutputGraph", 2, 
		{P_INT, P_TEXT, 0} },
        { "htgDestroyGraph", NULL, "_htgDestroyGraph", 1, 
		{P_INT, 0} },
        { "htgSetGraphTitle", NULL, "_htgSetGraphTitle", 6, 
		{P_INT, P_TEXT, P_INT, P_INT, P_INT, P_INT, 0} },
        { "htgSetGraphRange", NULL, "_htgSetGraphRange", 3, 
		{P_INT, P_INT, P_INT, 0} },
        { "htgSetFlag", NULL, "_htgSetFlag", 2, 
		{P_INT, P_INT, 0} },
        { "htgCreateLegend", NULL, "_htgCreateLegend", 4, 
		{P_INT, P_INT, P_INT, P_INT, 0} },

	/* Data management routines */
        { "htgAddDataSeries", NULL, "_htgAddDataSeries", 6, 
		{P_INT, P_TEXT, P_ARRAY, P_INT, P_INT, P_INT, 0} },
        { "htgAddTicks", NULL, "_htgAddTicks", 3, 
		{P_INT, P_ARRAY, P_INT, 0} },

	/* Graphing routines */
        { "htgPlotLineGraph", NULL, "_htgPlotLineGraph", 1, 
		{P_INT, 0} },

	{ NULL, 0 }
};



/*
** Types and macros of the Graph library
*/

typedef	struct _htgdata {
	int	red, green, blue,
		numValues,
		*data;
	char	*name;
	struct	_htgdata *next;
} htgData;



typedef struct {
	char	*title;
	int	borderType,
		borderWidth,
		align,
		font;
} htgTitle;



typedef struct {
	int	numTicks;
	char	**ticks;
} htgTick;




typedef struct {
	gdImage	*img;			/* GD Image Pointer */
	int	borderType,		/* Type of border if any */
		borderWidth,		/* Width of the border itself */
		width,			/* Width of the entire image */
		height,			/* Height of the entire image */
		cX,			/* X coord of canvas */
		cY,			/* Y coord of canvas */
		cWidth,			/* Width of the actual canvas */
		cHeight,		/* Height of the actual canvas */
		bgCol,			/* Background colour */
		bgRed,
		bgGreen,
		bgBlue,
		fgCol,			/* Foreground colour */
		minValue,		/* User defined MIN for scale */
		maxValue,		/* User defined MAX for scale */
		flags;
	htgData	*data;			/* Plot data */
	htgTick *xTicks,		/* Graph ticks (x axis)*/
		*yTicks;		/* Graph ticks (y axis)*/
	htgTitle *title;		/* Graph title */
} htgGraph;


#define	HTG_BORDER_NONE		0
#define	HTG_BORDER_BEVEL_IN	1
#define	HTG_BORDER_BEVEL_OUT	2
#define	HTG_BORDER_FRAME_IN	3
#define	HTG_BORDER_FRAME_OUT	4
#define	HTG_BORDER_BOX		5

#define	HTG_X_AXIS		1
#define	HTG_Y_AXIS		2

#define	HTG_ALIGN_LEFT		0
#define HTG_ALIGN_RIGHT		1
#define HTG_ALIGN_CENTER	2

#define	HTG_FONT_TINY		1
#define	HTG_FONT_SMALL		2
#define	HTG_FONT_MEDIUM_BOLD	3
#define	HTG_FONT_LARGE		4
#define	HTG_FONT_GIANT		5

#define	HTG_LINE_GRAPH		1
#define HTG_BAR_GRAPH		2

#define HTG_DASH_MARKS		1
